<?php

class Objective2Controller extends \BaseController
{

	/**
	 * Display a listing of the resource.
	 *
	 * @return Response
	 */

	public function index()
	{

		$title = "Learning Outcomes and Criteria";
		$outcomes = Outcome::orderBy('name', 'ASC')->get();
		$schools = School::orderBy('name', 'ASC')->get();
		$objectives = Objective::withTrashed()->orderBy('text', 'ASC')->get();

		return View::make('global.view-learning-outcomes-criteria', compact('title', 'outcomes', 'schools', 'objectives'));
	}



	/**
	 * Show the form for creating a new resource.
	 *
	 * @return Response
	 */
	public function isObjectiveUnique($input, $existing_Objective = NULL)
	{

		Log::info('isObjectiveUnique');

		if (Input::get('program_id') != 0)
			$program_id = $input['program_id'];
		else
			$program_id = NULL;

		$saved_Objective = Objective::withTrashed()
			->where('text', '=', $input['text'])
			->where('outcome_id', '=', $input['outcome_id'])
			->where('program_id', '=', $program_id)

			->first();



		if ($saved_Objective)
			return false;
		else
			return true;
	}


	private function makeValidator($clean_input)
	{
		/** Validation rules */
		return Validator::make(
			array(
				'text' => $clean_input['text'],

				'outcome_id' => $clean_input['outcome_id'],
				'program_id' => $clean_input['program_id']

			),
			array(
				'text' => 'required|string',

				'outcome_id' => 'required|array',
				'program_id' => 'required|numeric|integer'

			)

		);
	}


	private function cleanInput()
	{
		$clean_input = array();

		$clean_input['text'] = trim(preg_replace('/\t+/', '', Input::get('text')));


		$clean_input['outcome_id'] = Input::get('outcome');
		$counter = Input::get('counter') + 0;

		for ($i = 0; $i < $counter; $i++) {
			$clean_input['outcome_id'][$i] = trim(preg_replace('/\t+/', '', $clean_input['outcome_id'][$i]));
		}

		$clean_input['program_id'] = trim(preg_replace('/\t+/', '', Input::get('program_id')));


		return $clean_input;
	}

	private function cleanAssocInput()
	{
		$clean_input = array();

		$clean_input['text'] = trim(preg_replace('/\t+/', '', Input::get('text')));


		$clean_input['outcome_id'] = Input::get('assoc_outcome');
		$counter = Input::get('counter') + 0;

		for ($i = 0; $i < $counter; $i++) {
			$clean_input['outcome_id'][$i] = trim(preg_replace('/\t+/', '', $clean_input['outcome_id'][$i]));
		}

		$clean_input['program_id'] = trim(preg_replace('/\t+/', '', Input::get('program_id')));


		return $clean_input;
	}
	public function fetchObjective()
	{
		return Objective::find(Input::get('id'));
	}

	public function fetchObjectiveWithTrashed()
	{
		return json_encode(DB::select('select o.id, o.program_id, o.text, outc.outcome_id outcome_id from objectives o, objective_outcome outc where o.id = ? and o.id = outc.objective_id', array(Input::get('id'))));
	}
	/**
	 * Create a new Objective.
	 *
	 * @return Redirect Redirect back to form page
	 */
	public function create()
	{

		$clean_input = $this->cleanInput();

		/** Validation rules */
		$validator = $this->makeValidator($clean_input);

		/** If validation fails */
		if ($validator->fails()) {
			/** Prepare error message */
			$message = '<p>Error(s) creating a new Objective:</p><ul>';

			foreach ($validator->messages()->all('<li>:message</li>') as $validationError) {
				$message .= $validationError;
			}

			$message .= '</ul>';

			/** Send error message and old data */
			Session::flash('status', 'danger');
			Session::flash('message', $message);
			return Redirect::to('objective')->withInput();
		} else {
			// Check Objective uniqueness
			if (!$this->isObjectiveUnique($clean_input)) {
				/** Send error message and old data */
				Session::flash('status', 'danger');
				Session::flash('message', "This objective is a duplicate of an already saved Objective because it's and associated program are the same.");
				return Redirect::to('objective')->withInput();
			}

			/** Instantiate new Objective */
			$objective = new Objective;
			$objective->text = $clean_input['text'];




			// Set program
			if (Input::get('program_id') != 0)
				$objective->program_id = $clean_input['program_id'];
			else
				$objective->program_id = NULL;


			/** If Objective is saved, send success message */
			if ($objective->save()) {
				$objectiveId = $objective->id;
				foreach ($clean_input['outcome_id'] as $outcome_id) {
					DB::insert("insert into `objective_outcome` (objective_id, outcome_id) values ({$objectiveId}, {$outcome_id})");

					/*if (!($objectiveOutcome->save())) {
						Session::flash('status', 'danger');
						Session::flash('message', '<p>Error creating objective. Please try again later.</p>');
						return Redirect::to('objective')->withInput();
					}*/
				}

				Session::flash('status', 'success');
				Session::flash('message', 'Objective created: "' . $objective->text . '".');
				return Redirect::to('objective')->withInput(Input::only('outcome_id'));
			}

			/** If saving fails, send error message and old data */
			else {
				Session::flash('status', 'danger');
				Session::flash('message', '<p>Error creating objective. Please try again later.</p>');
				return Redirect::to('objective')->withInput();
			}
		}
	}
	/**
	 * Store a newly created resource in storage.
	 *
	 * @return Response
	 */
	public function store()
	{
		//
	}


	/**
	 * Display the specified resource.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function show($id)
	{
		//
	}


	/**
	 * Show the form for editing the specified resource.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function edit()
	{
		$title = "Objective";
		$outcomes = Outcome::orderBy('name', 'ASC')->lists('name', 'id');

		$objectives = Objective::withTrashed()->orderBy('text', 'ASC')->get();
		$programs = Program::orderBy('name', 'ASC')->get();

		return View::make('local.managers.admins.objectives', compact('title', 'outcomes', 'objectives', 'programs'));
	}




	/**
	 * Update the specified resource in storage.
	 *
	 * @param  int  $id
	 * @return Response
	 */

	public function update()
	{
		$Objective = Objective::withTrashed()->find(Input::get('id'));

		$clean_input = $this->cleanAssocInput();

		/** Validation rules */
		$validator = $this->makeValidator($clean_input);

		/** If validation fails */
		if ($validator->fails()) {
			/** Prepare error message */
			$message = 'Error(s) updating the Objective: <ul>';

			foreach ($validator->messages()->all('<li>:message</li>') as $validationError) {
				$message .= $validationError;
			}

			$message .= '</ul>';

			/** Send error message and old data */
			Session::flash('status', 'danger');
			Session::flash('message', $message);
			return Redirect::back()->withInput();
		} else {



			/** Set info */
			$Objective->text = $clean_input['text'];



			// Set program
			if (Input::get('program_id') != 0)
				$Objective->program_id = Input::get('program_id');
			else
				$Objective->program_id = NULL;

			// Set status


			/** If Objective is updated, send success message */
			if ($Objective->save()) {

				$objectiveId = $Objective->id;
				DB::delete("delete from `objective_outcome` where objective_id ={$objectiveId}");

				foreach ($clean_input['outcome_id'] as $outcome_id) {
					DB::insert("insert into `objective_outcome` (objective_id, outcome_id) values ({$objectiveId}, {$outcome_id})");
					Session::flash('status', 'success');
					Session::flash('message', 'Updated Objective: "' . $Objective->text . '"');
					return Redirect::back();
				}
			}

			/** If saving fails, send error message and old data */
			else {
				Session::flash('status', 'danger');
				Session::flash('message', 'Error updating the Objective. Please try again later.');
				return Redirect::back()->withInput();
			}
		}
	}



	/**
	 * Remove the specified resource from storage.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function destroy($id)
	{
		//
	}
}
